#!/bin/sh
# 

d_FLAG="FALSE"
h_FLAG="FALSE"

CARM_ORACLEVERSION_FOUND="FALSE"
CARM_ORACLE_VERSION="10.0"

# CARM_ORACLE_INSTALLDIR
if [ "X$CARM_ORACLE_INSTALLDIR" == "X" ]; then 
  CARM_ORACLE_INSTALLDIR="/usr/lib/oracle"
fi


# Debug output (-d flag)
printDebug() {
  if [ X$DEBUG = "XTRUE" ]; then
    echo $1
  fi
}


# Print Help (-h flag)
printHelp() {
  echo ""
  echo "  This script searches for the latest version of Oracle Instant Client" 
  echo "  and sets the PATHs accordingly. Special treatment for different" 
  echo "  package formats between versions of Oracle has been implemented."
  echo ""
  echo "  * Set the environment variable:"
  echo "  export CARM_ORAINSTCLNT_PREFERRED_VERSIONS=\"... ...\""
  echo "  to define the version regular expression (defaults to 11.1* 10.2*)"
  echo "  Means that any version of Oracle 11.1 has preference over 10.2."
  echo ""
  echo "  * Optionally, set the environment variable:"
  echo "  export CARM_ORACLE_INSTALLDIR=\"/.../.../...\""
  echo "  (defaults to /usr/lib/oracle)"
  echo ""
  echo "Usage: $0 "
  echo "          [-h|--help] "
  echo "          [-d|--debug] "
  echo ""
  exit 0
}


# Process input arguments and options
while true; do
  case "$1" in
    -h|--help)
      printHelp
      h_FLAG="TRUE"
      shift 1
      ;;
    -d|--debug)
      DEBUG="TRUE"
      d_FLAG="TRUE"
      printDebug "Debug Mode ON"
      printDebug ""
      shift 1
      ;;
    *)
      break
      ;;
  esac
done


# CARM_ORAINSTCLNT_PREFERRED_VERSIONS
# Set Default versions, if none set from the SHELL
TMP_CARM_ORA_FIRSTVER=`echo $CARM_ORAINSTCLNT_PREFERRED_VERSIONS | awk '-F ' '{print $1}'`
if [ "X$TMP_CARM_ORA_FIRSTVER" == "X" ]; then
  CARM_ORAINSTCLNT_PREFERRED_VERSIONS="11.1* 10.2*"
  printDebug "=== CARM_ORAINSTCLNT_PREFERRED_VERSIONS=$CARM_ORAINSTCLNT_PREFERRED_VERSIONS"
  printDebug "(Using the DEFAULT preferred Oracle versions)"

else
  printDebug "=== CARM_ORAINSTCLNT_PREFERRED_VERSIONS=$CARM_ORAINSTCLNT_PREFERRED_VERSIONS"
  printDebug "(Using SHELL defined preferred Oracle versions)"

fi
printDebug ""


# In order of user selected prio, find the highest version 
# of Orcle InstantClient, for a requested version regexp (then break)
carm_ora_dirlist_all=""
for v in ${CARM_ORAINSTCLNT_PREFERRED_VERSIONS}; do

  # Find all matching file system elements
  # (Expand the wildcards. Prepend oracle install directory for each)
  carm_ora_dirlist_all="$CARM_ORACLE_INSTALLDIR/$v";
  printDebug "carm_ora_dirlist_all=$carm_ora_dirlist_all"

  # Make sure each element is an existing directory, otherwise mask out 
  # (Avoid errors, especially for elements without wildcards)
  for d in $carm_ora_dirlist_all; do
    if [ -d $d ]; then
      carm_ora_dirlist_short="$carm_ora_dirlist_short $d"
    fi
  done
  printDebug "carm_ora_dirlist_short=$carm_ora_dirlist_short"

  # For the shortlist of existing directories, sort them by version
  tmp_carm_ora_firstdir=`echo $carm_ora_dirlist_short | awk -F' ' '{print $1}'`
  if [ "X$tmp_carm_ora_firstdir" != "X" ]; then
    # \ls -d1 -v $carm_ora_dirlist_short; 
    # carm_ora_result=$?
    CARM_ORACLE_PATH=$(\ls -d1 -v $carm_ora_dirlist_short | tail -1) 
    carm_ora_result=0
  else 
    carm_ora_result=1
  fi

  # Reset control variables
  unset tmp_carm_ora_firstdir
  unset carm_ora_dirlist_all
  unset carm_ora_dirlist_short

  if [ $carm_ora_result -eq 0 ] && [ "X$CARM_ORACLE_PATH" != "X" ]; then
    CARM_ORACLE_VERSION=`basename $CARM_ORACLE_PATH`
    break
  fi

done
printDebug ""

printDebug "CARM_ORACLE_PATH=$CARM_ORACLE_PATH"
printDebug "CARM_ORACLE_VERSION=$CARM_ORACLE_VERSION"
printDebug ""

if [ $carm_ora_result -ne 0 ] || [ "X$CARM_ORACLE_VERSION" == "X" ]; then
  echo "Carmen oracle-instantclient start script: Oracle version NOT found."
fi

# Cleanup a bit
unset tmp_carm_ora_firstdir
unset carm_ora_dirlist_all
unset carm_ora_dirlist_short


# Find latest version with 'ls'
if [ X$CARM_ORACLE_VERSION != X ]; then

  # Some sanity checking (must be a directory)
  if [ -d $CARM_ORACLE_INSTALLDIR/$CARM_ORACLE_VERSION ]; then

    ### Check for newer Oracle package format first (v11.1.0.7.0 and above)
    # - We cannot assume a fixed packagename for oracle-instantclient.  
    #   A version suffix appended, which is just the entity we are looking for.
    # Some sanity checking again (properly named package installed)
    rpm -q oracle-instantclient$CARM_ORACLE_VERSION-basic 2>&1 > /dev/null
    if [ $? -eq 0 ]; then
      CARM_ORACLEVERSION_FOUND="TRUE"
      _ORACLE_VERSION=$CARM_ORACLE_VERSION
    fi

    ### Check for older Oracle package format (version 11.1.0.6.0 and below)
    # - Package has a fixed name, without version number in the name
    if [ X$CARM_ORACLEVERSION_FOUND != XTRUE ]; then 
      CARM_ORACLE_VERSION=$(rpm --queryformat="%{version} " -q oracle-instantclient-basic) 2>&1 > /dev/null
      if [ $? -eq 0 ]; then
        CARM_ORACLEVERSION_FOUND="TRUE"
        _ORACLE_VERSION=$(echo $CARM_ORACLE_VERSION | cut -d " " -f1)
      fi
    fi
  fi
fi


# Assuming the Oracle version has been found, set the PATHs
if [ X$CARM_ORACLEVERSION_FOUND == XTRUE ]; then 

  #this is made to avoid problems when more than one package with the same name
  #is installed, like a package for both x86_64 and i386 or two different
  #releases of the same package
  if [ "`/bin/uname -m`" = x86_64 ]; then
    _ORACLE_CLIENT_PATH="/usr/lib/oracle/$_ORACLE_VERSION/client64"
  else
    _ORACLE_CLIENT_PATH="/usr/lib/oracle/$_ORACLE_VERSION/client"
  fi

  if [ -d "$_ORACLE_CLIENT_PATH/bin" ]; then
    if [ -n "$PATH" ]; then
      PATH=$_ORACLE_CLIENT_PATH/bin:$PATH
    else
      PATH=$_ORACLE_CLIENT_PATH/bin
    fi
    export PATH
  fi

  if [ -d "$_ORACLE_CLIENT_PATH/lib" ]; then
    if [ -n "$LD_LIBRARY_PATH" ]; then
      LD_LIBRARY_PATH=$_ORACLE_CLIENT_PATH/lib:$LD_LIBRARY_PATH
    else
      LD_LIBRARY_PATH=$_ORACLE_CLIENT_PATH/lib
    fi
    export LD_LIBRARY_PATH
  fi

  # --- Debug printouts
  printDebug "----------------------------------------------"
  printDebug ""
  printDebug "=== _ORACLE_VERSION = $_ORACLE_VERSION"
  printDebug "=== _ORACLE_CLIENT_PATH = $_ORACLE_CLIENT_PATH"
  printDebug ""
  printDebug "=== PATH = $PATH"
  printDebug "=== LD_LIBRARY_PATH = $LD_LIBRARY_PATH"
  printDebug ""

  unset _ORACLE_CLIENT_PATH

fi

unset _ORACLE_VERSION


